#include <stdio.h>
#include <kernel.h>
#include <draw.h>
#include <graph.h>
#include <malloc.h>
#include <libcdvd.h>
#include <libpad.h>

#include "mandel_calc.h"

#define TOTAL_TRIANGLES 3
#define MIN_BOUNDARY -20
#define MAX_BOUNDARY 20

#define TEXTURE_WIDTH 256
#define TEXTURE_HEIGHT 256
#define TEXTURE_DEPTH 4

int rect_points_count=6;
int rect_vertex_count=4;

int rect_points[6] = { 0, 1, 2, 0, 3, 2 };

VECTOR rect_vertices[] =
{
  -30.00f,  30.00f, 0.00f, 1.00f,
   30.00f,  30.00f, 0.00f, 1.00f,
   30.00f, -30.00f, 0.00f, 1.00f,
  -30.00f, -30.00f, 0.00f, 1.00f
};

VECTOR rect_colors[] =
{
  1.00f, 1.00f, 1.00f, 1.00f,
  1.00f, 1.00f, 1.00f, 1.00f,
  1.00f, 1.00f, 1.00f, 1.00f,
  1.00f, 1.00f, 1.00f, 1.00f
};

VECTOR rect_position = { 0.0f, 0.0f, 0.0f, 1.0f };
VECTOR rect_rotation = { 0.0f, 0.0f, 0.0f, 1.0f };

VECTOR texture_coordinates[] =
{
  1.00f, 0.00f, 0.00f, 0.00f,
  0.00f, 0.00f, 0.00f, 0.00f,
  0.00f, 1.00f, 0.00f, 0.00f,
  1.00f, 1.00f, 0.00f, 0.00f
};

VECTOR camera_position = { 0.00f, 0.00f, 100.00f, 1.00f };
VECTOR camera_rotation = { 0.00f, 0.00f,   0.00f, 1.00f };

int main (void)
{
MATRIX local_world;
MATRIX world_view;
MATRIX view_screen;
MATRIX local_screen;
VECTOR *temp_vertices;
u64 *rect_xyz;
u64 *rect_rgbaq;
u64 *rect_st;
u8 *texture_data;
u32 texture_len;
int t,j;
FILE *in;
struct padButtonStatus mypad;
u8 *padarea;
struct mandelbrot_t mandelbrot;
int rotate=1;
float diff;

  mandelbrot.width=TEXTURE_WIDTH;
  mandelbrot.height=TEXTURE_HEIGHT;
  mandelbrot.real_start=-2.00;
  mandelbrot.real_end=1.00;
  mandelbrot.imaginary_start=-1.00;
  mandelbrot.imaginary_end=1.00;
  mandelbrot.count=100;

  temp_vertices = memalign(128, sizeof(VECTOR) * rect_vertex_count);

  rect_xyz   = memalign(128, sizeof(u64) * rect_vertex_count);
  rect_rgbaq = memalign(128, sizeof(u64) * rect_vertex_count);
  rect_st = memalign(128, sizeof(u64) * rect_vertex_count);

  draw_initialize(GRAPH_MODE_AUTO, GRAPH_PSM_32, GRAPH_PSM_32);
  // graph_initialize();

  create_view_screen(view_screen, graph_get_aspect(), -3.00f, 3.00f, -3.00f, 3.00f, 1.00f, 2000.00f);

  texture_len=TEXTURE_WIDTH*TEXTURE_HEIGHT*TEXTURE_DEPTH;
  texture_data=memalign(128,texture_len);
  memset(texture_data,0,texture_len);

  mandel_calc((u32 *)texture_data,&mandelbrot);

  graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);

  cdStop();

  SifLoadModule("rom0:SIO2MAN",0,NULL);
  SifLoadModule("rom0:PADMAN",0,NULL);
  padInit(0);
  padarea = memalign(64, 256);
  memset(padarea,0,256);
  memset(&mypad,0,sizeof(struct padButtonStatus));
  padPortOpen(0,0,padarea);

  while(padGetState(0,0)!=PAD_STATE_STABLE);

  padSetMainMode(0,0,PAD_MMODE_DUALSHOCK,PAD_MMODE_LOCK);
  padEnterPressMode(0,0);

  while(1)
  {
    create_world_view(world_view, camera_position, camera_rotation);

    padRead(0,0,&mypad);
/*
    if (mypad.ljoy_h!=0)
    {
      rect_rotation[1] += 0.100;
      while (rect_rotation[1] > 3.14f) { rect_rotation[1] -= 6.28f; }
    }
*/

    if ((mypad.btns&PAD_LEFT)==0)
    {
      rect_rotation[1] -= 0.100;
      while (rect_rotation[1] < 3.14f) { rect_rotation[1] += 6.28f; }
    }

    if ((mypad.btns&PAD_RIGHT)==0)
    {
      rect_rotation[1] += 0.100;
      while (rect_rotation[1] > 3.14f) { rect_rotation[1] -= 6.28f; }
    }

    if ((mypad.btns&PAD_UP)==0)
    {
      rect_rotation[0] -= 0.100;
      while (rect_rotation[0] < -3.14f) { rect_rotation[0] += 6.28f; }
    }

    if ((mypad.btns&PAD_DOWN)==0)
    {
      rect_rotation[0] += 0.100;
      while (rect_rotation[0] > 3.14f) { rect_rotation[0] -= 6.28f; }
    }

    if ((mypad.ljoy_h)>200)
    {
      rect_rotation[2] += ((0.100)/55)*(mypad.ljoy_h-200);
      while (rect_rotation[2] > 3.14f) { rect_rotation[2] -= 6.28f; }
    }

    if ((mypad.ljoy_h)<55)
    {
      rect_rotation[2] -= ((0.100)/55)*(55-mypad.ljoy_h);
      while (rect_rotation[2] < -3.14f) { rect_rotation[2] += 6.28f; }
    }

    if ((mypad.rjoy_h)>200)
    {
      diff=((0.100)/55)*(mypad.ljoy_h-200);
      mandelbrot.real_start+=diff;
      mandelbrot.real_end+=diff;
      mandel_calc((u32 *)texture_data,&mandelbrot);
      graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);
    }

    if ((mypad.rjoy_h)<55)
    {
      diff=((0.100)/55)*(55-mypad.ljoy_h);
      mandelbrot.real_start-=diff;
      mandelbrot.real_end-=diff;
      mandel_calc((u32 *)texture_data,&mandelbrot);
      graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);

    }

    if ((mypad.rjoy_v)>200)
    {
      diff=((0.100)/55)*(mypad.ljoy_h-200);
      mandelbrot.imaginary_start-=diff;
      mandelbrot.imaginary_end-=diff;
      mandel_calc((u32 *)texture_data,&mandelbrot);
      graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);
    }

    if ((mypad.rjoy_v)<55)
    {
      diff=((0.100)/55)*(55-mypad.ljoy_h);
      mandelbrot.imaginary_start+=diff;
      mandelbrot.imaginary_end+=diff;
      mandel_calc((u32 *)texture_data,&mandelbrot);
      graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);
    }

    if ((mypad.btns&PAD_CROSS)==0)
    {
      diff=0.100;
      mandelbrot.imaginary_start+=diff;
      mandelbrot.imaginary_end-=diff;
      mandelbrot.real_start+=diff;
      mandelbrot.real_end-=diff;
      mandel_calc((u32 *)texture_data,&mandelbrot);
      graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);

    }

/*
    if (rotate==1)
    {
      rect_rotation[0] += 0.050;
      while (rect_rotation[0] > 3.14f) { rect_rotation[0] -= 6.28f; }
    }
*/

    create_local_world(local_world, rect_position, rect_rotation);
    create_local_screen(local_screen, local_world, world_view, view_screen);

    calculate_vertices(temp_vertices, 4, rect_vertices, local_screen);
    draw_generate_xyz(rect_xyz, 4, temp_vertices);
    draw_generate_rgbaq(rect_rgbaq, 4, temp_vertices, rect_colors);
    draw_generate_st(rect_st, 4, temp_vertices, texture_coordinates);

/*
    mandel_calc((u32 *)texture_data,&mandelbrot);

    graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32, texture_data, texture_len);
*/

    graph_wait_vsync();

    draw_swap();
    draw_clear(0,0,0);

    graph_set_texture(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_32);
    draw_triangles_textured(rect_points, 6, rect_xyz, rect_rgbaq, rect_st);
    /* draw_triangles(rect_points, 6, rect_xyz, rect_rgbaq); */

  }

  return 0;
}

